/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */

package org.objectweb.medor.api;

import java.io.PrintStream;
import java.io.PrintWriter;

/**
 * This class describes exceptions raised if any problem occurs
 * in MEDOR.
 *
 * @author Sebastien Chassande-Barrioz
 */
public class MedorException extends Exception {

	Exception inner = null;

	public MedorException(String s) {
		super(s);
	}

	public MedorException(Exception e) {
		super("Nested Exception");
		inner = e;
	}

	public MedorException(String m, Exception e) {
		super(m);
		inner = e;
	}

	public Exception getNestedException() {
		return inner;
	}

	public void printStackTrace() {
		printStackTrace(System.out);
	}

	public void printStackTrace(PrintWriter pw) {
		if (inner!=null) {
			pw.println(super.getMessage());
			inner.printStackTrace(pw);
		}
		else {
			super.printStackTrace(pw);
		}
	}
	public void printStackTrace(PrintStream ps) {
		if (inner!=null) {
			ps.println(super.getMessage());
			inner.printStackTrace(ps);
		}
		else {
			super.printStackTrace(ps);
		}
	}


}
