/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.datasource.api;

import org.objectweb.medor.api.DataSourceException;

/**
 * A WrapperFactory is called at evaluation time by the QueryEvaluator when
 * reaching a QueryLeaf. The WrapperFactory generates the appropriate
 * Wrapper for the QueryLeaf.
 * It maintains its pool of Wrappers.
 *
 * @author Sebastien Chassande-Barrioz
 */
public interface WrapperFactory {

	Wrapper getWrapper(DataStore dl) throws DataSourceException;

	/**
	 * For a given data store type, associates a Wrapper class.
	 * This will be used by the evaluator to associate the right Wrapper to
	 * each query leaf (data store).
	 * The Wrapper class must have an empty constructor (because it is
	 * instanciated using newInstance).
	 * @param dataStoreType is the type of data store
	 * (DataStore.getDataStoreType())
	 * @param wrapperClass is the java Class for the associated Wrapper.
	 */
	void bindDataStoreToWrapper(short dataStoreType,
								java.lang.Class wrapperClass)
		throws DataSourceException;
}
