/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.datasource.lib;

import org.objectweb.medor.api.DataSourceException;
import org.objectweb.medor.datasource.api.DataStore;

/**
 * This implementation of the DataStore interface could be used in all cases
 * where a data store could be identified by a connection factory. The reference
 * is not typed in order to support various connection factory (
 * javax.sql.Datasource, javax.resource.cci.ConnectionFactory, ...)
 *
 * @author S.Chassande-Barrioz
 */
public class ConnectionFactoryDataStore extends BasicDataStore {
	protected Object connectionFactory;

    /**
     * It builds a ConnectionFactoryDataStore.
     * @param dataStoreType is the type of the DataStore
     * (ex: DataStore.JDBC_STORE)
     * @param sourceName is the name of the datastore
     * @param capabilities is the capabilities of the datastore
     * @param cf is the connection factory (cannot be null)
     * @throws DataSourceException if the specified connection factory is null.
     */
    public ConnectionFactoryDataStore(short dataStoreType,
						  String sourceName,
						  short[] capabilities,
						  Object cf)
            throws DataSourceException {
		super(dataStoreType, sourceName, capabilities);
		if (cf == null)
			throw new DataSourceException("Invalid connection factory reference");
		connectionFactory = cf;
	}

	public Object getConnectionFactory() {
		return connectionFactory;
	}

	/**
	 * Two ConnectionFactoryDataStore with the same connection factory are equals.
	 */
	public boolean isSameAs(DataStore ds) {
		return (ds instanceof ConnectionFactoryDataStore)
				&& ((ConnectionFactoryDataStore) ds).connectionFactory.equals(connectionFactory);
	}
}
