/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.eval.api;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.query.api.QueryNode;

/**
 * This interface creates the controller object for the evaluation of a given
 * QueryNode within a QueryTree.
 */
public interface NodeEvaluatorFactory {

    /**
     * This methods creates a NodeEvaluator.
     * It chooses the right implementation depending on the type of operation,
     * the OptimizationMetaData associated to the QueryNode.
     * It does not create NodeEvaluators for the children QueryNodes of the
     * parameter QueryNode. It is up to the Evaluator to call the Factory
     * for creating NodeEvaluators for the subnodes.
     * @param node: the QueryNode for which the NodeEvaluator is created
     * @param subEvaluators: the NodeEvaluators for the children QueryTrees
     * of the QueryNode.
     */
    NodeEvaluator createNodeEvaluator(QueryNode node,
                                             NodeEvaluator[] subEvaluators)
            throws MedorException;

}
