/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.eval.cache.api;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.tuple.api.Tuple;

import java.util.Iterator;

/**
 * This interface represents a cache for TupleCollection objets.
 */
public interface CollectionCache {

	/**
	 * Are insertions possible (is policy dependent: example may require
	 * to free the cache before insertion).
	 */
	boolean isCanInsert();

    /**
	 * This method allow stop or no inserting objetcs in the cache.
	 */
	void setCanInsert(boolean canInsert);

    /**
	 * @param tupleIndex is the rank of the Object
	 */
	Tuple getTuple(int tupleIndex) throws MedorException;

    boolean putTuple(int indexTuple, Tuple t) throws MedorException;

    /**
	 * Does the cache contains Object ranked int of the associated
	 * Objects to be cached.
	 */
	boolean contains(int index);

	long getCapacity();

    /**
	 * Creates the internal structure of the cache.
	 */
	void initialize();

	/**
	 * Returns an iterator over the indexes of the Objects which are present
	 * in the cache.
	 */
    Iterator tupleIndexIterator();

	/**
	 * Reinitializes the internal structure of the cache.
	 */
	//TODO renommer en reset.
    void destroy();

}
