/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.eval.lib;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.QueryNodeException;
import org.objectweb.medor.eval.api.NodeEvaluator;
import org.objectweb.medor.eval.cache.lib.TupleCache;
import org.objectweb.medor.eval.cache.lib.WindowCachedTupleCollection;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.query.api.NestQueryNode;
import org.objectweb.medor.query.api.OperationType;
import org.objectweb.medor.query.api.QueryNode;
import org.objectweb.medor.query.api.UnnestQueryNode;
import org.objectweb.medor.tuple.api.TupleCollection;

public class UnaryIteratifNodeEvaluator implements NodeEvaluator {

	QueryNode query;
	NodeEvaluator subNodeEvaluator;
	long cacheSize;
	boolean open = false;
    PrefetchBuffer prefetchBuffer;

	public UnaryIteratifNodeEvaluator(QueryNode query,
									  NodeEvaluator subNodeEvaluator) {
		this.query = query;
		this.subNodeEvaluator = subNodeEvaluator;
	}

	public boolean linkChildren() {
		// TO DO
		open = true;
		return true;
	}

	public boolean unlinkChildren() {
		// TO DO
		open = false;
		return true;
	}

	public TupleCollection fetchData(ParameterOperand[] parameters)
		throws MedorException {
		TupleCollection res = null;
        switch (query.getType()) {
		case OperationType.NEST:
            res = new NestEvaluatedTC(
                    (NestQueryNode) query, subNodeEvaluator, parameters);
            break;
		case OperationType.PROJECT:
            res = new ProjectEvaluatedTC(query, subNodeEvaluator, parameters, prefetchBuffer);
            break;
		case OperationType.SELECT_PROJECT:
            if (query.getQueryFilter() == null) {
                res = new ProjectEvaluatedTC(query, subNodeEvaluator, parameters, prefetchBuffer);
            } else {
                res = new SelProjEvaluatedTC(query, subNodeEvaluator, parameters, prefetchBuffer);
            }
            break;
		case OperationType.UNNEST:
            res = new UnnestEvaluatedTC(
                    (UnnestQueryNode) query, subNodeEvaluator, parameters);
            break;
		default:
			throw new QueryNodeException("Unreconized QueryNode: " + query);
		}
        if (query.getOptimizationMetaData() == null) {
            // No optimization
            return res;
        } else if (query.getOptimizationMetaData().isCache()) {
            return new WindowCachedTupleCollection(
                    res, new TupleCache(cacheSize));
        } else {
            return res;
        }
	}

    public void setPrefetchBuffer(PrefetchBuffer pb) throws MedorException {
        prefetchBuffer = pb;
    }

	public long getCacheSize() {
		return cacheSize;
	}

	public void setCacheSize(long cacheSize) {
		this.cacheSize = cacheSize;
	}

	public QueryNode getQueryNode() {
		return query;
	}
}
