/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */
package org.objectweb.medor.filter.jorm.lib;

import org.objectweb.jorm.naming.api.PName;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.CalculatedParameterOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.type.lib.QType;

/**
 * This implementation of the CalculatedParameterOperand interface checks if
 * a PName is null. This PName must be given in parameter among the
 * ParameterOperand of the evaluate method.
 *
 * @author A. Lefebvre
 */
public class IsNullPName
	extends BasicParameterOperand
	implements CalculatedParameterOperand {


	public IsNullPName(String parameterName) {
		super(PTypeSpaceMedor.BOOLEAN, parameterName);
	}

	// IMPLEMENTATION OF THE CalculatedParameterOperand INTERFACE //
	//------------------------------------------------------------//

	public void evaluate(ParameterOperand[] pos) throws ExpressionException {
		PName pn = null;
		for (int i = 0; i < pos.length; i++) {
			if (pos[i] != null
                    && name.equals(pos[i].getName())
				    && pos[i].getType().getTypeCode() == QType.TYPECODE_PNAME) {
				pn = (PName) pos[i].getObject();
			}
		}
		if (pn == null) {
			throw new ExpressionException("Impossible to evaluate this parameter wihtout the parameter " + name);
		}
        this.setValue(pn.isNull());
    }
}
