/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, S. Chassande-Barrioz, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.lib.BasicVariableOperand;
import org.objectweb.jorm.type.api.PTypeSpace;

/**
 * Avg aggregate function.
 * <p>
 * Avg applies to :
 * <ol>
 * <li>a FieldOperand, which is part of a NestedField.</li>
 * <li>a Collection of values (TupleCollection), specifying which Field
 * is to be aggregated</li>
 * <li>a Collection of values (Java Collection)</li>
 * </ol>
 * @author Alexandre Lefebvre
 */
public class Avg extends BasicAggregateOperator {

	public Avg() {
		super(PTypeSpace.FLOAT);
	}

	public Avg(Expression e) {
		super(PTypeSpace.FLOAT, e);
	}

	public Avg(Expression e, boolean distinct) {
		super(PTypeSpace.FLOAT, e, distinct);
	}

	public String getOperatorString() {
		return Operator.AVG;
	}

	public Operand compileExpression()
			throws ExpressionException, MalformedExpressionException {
		if (expressions[0] != null) {
			expressions[0].compileExpression();
				result = new BasicVariableOperand(PTypeSpace.FLOAT);
				verified = true;
		}
		else
		// Operands not yet assigned
			throw new MalformedExpressionException("null children value");
		return result;
	}

	public Operand evaluate(ParameterOperand[] pos, Object o)
			throws ExpressionException {
		throw new ExpressionException("Implementation of Avg by MEDOR not yet implemented");
	}
}
