/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.lib.BasicVariableOperand;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.type.lib.QTypeTuple;

import java.util.Map;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class BasicFieldOperand
	extends BasicVariableOperand
	implements FieldOperand {

	Field field = null;
	int index = -1;

	public BasicFieldOperand() {
	}

	public BasicFieldOperand(Field field) {
		super(field.getType());
		this.field = field;
	}

	public BasicFieldOperand(PType type, Field field) {
		super(type);
		this.field = field;
	}

	public Object clone(Object clone, Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		((BasicFieldOperand) clone).index = index;
		((BasicFieldOperand) clone).field = (Field) getClone(field, obj2clone);
		return clone;
	}

	public Field getField() {
		return field;
	}

	public void setField(Field f) {
		field = f;
		type = field.getType();
	}

	public int getIndex() {
		return index;
	}

	public void setIndex(int idx) {
		index = idx;
	}

	public String toString() {
		return "FieldOperand(" + (field == null ? "null" : field.getName() + ")");
	}

	public Operand evaluate(ParameterOperand[] pos, Object t)
		throws ExpressionException {
		Tuple o = (Tuple) t;
		try {
			switch (type.getTypeCode()) {
			case QTypeTuple.TYPECODE_BOOLEAN:
				setValue(o.getBoolean(index));
				break;
			case QTypeTuple.TYPECODE_BYTE:
				setValue(o.getByte(index));
				break;
			case QTypeTuple.TYPECODE_CHAR:
				setValue(o.getChar(index));
				break;
			case QTypeTuple.TYPECODE_DATE:
				setValue(o.getDate(index));
				break;
			case QTypeTuple.TYPECODE_DOUBLE:
				setValue(o.getDouble(index));
				break;
			case QTypeTuple.TYPECODE_FLOAT:
				setValue(o.getFloat(index));
				break;
			case QTypeTuple.TYPECODE_INT:
				setValue(o.getInt(index));
				break;
			case QTypeTuple.TYPECODE_LONG:
				setValue(o.getLong(index));
				break;
			case QTypeTuple.TYPECODE_SHORT:
				setValue(o.getShort(index));
				break;
			case QTypeTuple.TYPECODE_STRING:
				setValue(o.getString(index));
				break;
			case QTypeTuple.TYPECODE_OBJBOOLEAN:
			case QTypeTuple.TYPECODE_OBJBYTE:
			case QTypeTuple.TYPECODE_OBJCHAR:
			case QTypeTuple.TYPECODE_OBJDOUBLE:
			case QTypeTuple.TYPECODE_OBJFLOAT:
			case QTypeTuple.TYPECODE_OBJINT:
			case QTypeTuple.TYPECODE_OBJLONG:
			case QTypeTuple.TYPECODE_OBJSHORT:
			case QTypeTuple.TYPECODE_BIGDECIMAL:
			case QTypeTuple.TYPECODE_BIGINTEGER:
				setValue(o.getObject(index));
				break;
			}
		}
		catch (MedorException e) {
			throw new ExpressionException(e);
		}
		super.evaluate(pos, o);
		return this;
	}
}
