/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.lib.BasicBinaryOperator;
import org.objectweb.medor.expression.lib.BasicVariableOperand;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

/**
 * The InCollection class is an operator testing whether the value of a
 * Field (the left operand, which is supposed to be a FieldOperand) is equal to
 * one of the elements of a collecton (the right Operand)
 */
public class InCollection extends BasicBinaryOperator {

	/**
	 * elemType is the type of the elements of the collection
	 */
	private PType elemType;


	public InCollection(FieldOperand left, Operand right,
						PType elemType) {
		super(PTypeSpaceMedor.BOOLEAN, left, right);
		this.elemType = elemType;
	}

	public Object clone(Object clone, Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		((InCollection) clone).elemType = elemType;
		return clone;
	}

	/**
	 * Returns the declared PType of the elements.
	 */
	public PType getElemType() {
		return elemType;
	}

	public Operand evaluate(ParameterOperand[] pos, Object o)
		throws ExpressionException {
		try {
			expressions[0].evaluate(pos, o);
			Collection c = null;
			if (expressions[1] instanceof ParameterOperand) {
				String pn = ((ParameterOperand) expressions[1]).getName();
				if (pos == null || pos.length == 0)
					throw new ExpressionException("No Parameter");
				int i = 0;
				while (i < pos.length && !pn.equals(pos[i].getName()))
					i++;
				if (i < pos.length)
					c = (Collection) pos[i].getObject();
				else
					throw new ExpressionException("Parameter " + pn + " not found");
			} else
				c = (Collection) ((Operand) expressions[1]).getObject();
			Iterator it = c.iterator();
			boolean in = false;
			Object obj = ((FieldOperand)expressions[0]).getObject();
			while (it.hasNext() && !in)
				in = it.next().equals(obj);
			result.setValue(in);
		}
		catch (NullPointerException e) {
			// result == null or missing children objects
			throw new IllegalStateException("Unevaluable Expression: Not compiled");
		}
		return result;
	}

	public Operand compileExpression()
		throws TypingException, MalformedExpressionException {
		if ((expressions[0] != null) && (expressions[0] != null)) {
			result = new BasicVariableOperand(type);
			verified = true;
		}
		else
		// Operands not yet assigned
			throw new MalformedExpressionException("null children value");
		return result;
	}

	public String getOperatorString() {
		return Operator.IN;
	}
}
