/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.filter.lib;

import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.api.UnaryOperator;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.lib.BasicUnaryOperator;
import org.objectweb.medor.expression.lib.BasicVariableOperand;
import org.objectweb.medor.expression.type.ExpressionTypeHelper;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.type.lib.QTypeTuple;

import java.util.Map;

/**
 * IsNull operates on an Expression. Its evaluation returns true if the
 * Expression is null, false otherwise.
 */
public class IsNull extends BasicUnaryOperator implements UnaryOperator {

	boolean not = false;

    public IsNull() {
		super(PTypeSpaceMedor.BOOLEAN);
	}

	public IsNull(Expression e) {
		super(PTypeSpaceMedor.BOOLEAN, e);
	}

	public IsNull(Expression e, boolean not) {
		super(PTypeSpaceMedor.BOOLEAN, e);
        this.not = not;
	}

	public Object clone(Object clone, Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		((IsNull) clone).not = not;
		return clone;
	}

    public void setNot(boolean not) {
        this.not = not;
    }

    public boolean getNot() {
        return not;
    }

	public org.objectweb.medor.expression.api.Operand evaluate(ParameterOperand[] pos, Object o)
		throws ExpressionException {
		try {
			Operand subResult = expressions[0].evaluate(pos, o);
			switch (subResult.getType().getTypeCode()) {
			case QTypeTuple.TYPECODE_OBJBOOLEAN:
			case QTypeTuple.TYPECODE_OBJCHAR:
			case QTypeTuple.TYPECODE_OBJBYTE:
			case QTypeTuple.TYPECODE_OBJSHORT:
			case QTypeTuple.TYPECODE_OBJINT:
			case QTypeTuple.TYPECODE_OBJLONG:
			case QTypeTuple.TYPECODE_OBJFLOAT:
			case QTypeTuple.TYPECODE_OBJDOUBLE:
			case QTypeTuple.TYPECODE_BIGDECIMAL:
			case QTypeTuple.TYPECODE_BIGINTEGER:
			case QTypeTuple.TYPECODE_STRING:
			case QTypeTuple.TYPECODE_DATE:
			case QTypeTuple.TYPECODE_CHARARRAY:
			case QTypeTuple.TYPECODE_BYTEARRAY:
			case QTypeTuple.TYPECODE_SERIALIZED:
			case QTypeTuple.TYPECODE_REFERENCE:
			case QTypeTuple.TYPECODE_TUPLE:
			case QTypeTuple.TYPECODE_TUPLE_COLLECTION:
				result.setValue(evaluate(subResult.getObject()));
				break;
			default:
				throw new MalformedExpressionException(
					"Unauthorized expression element");
			}
		}
		catch (NullPointerException e) {
			throw new ExpressionException("Unevaluable Expression: Not compiled");
		}
		return result;
	}

	public boolean evaluate(Object op) {
		return (not ? op != null : op == null);
	}

	public Operand compileExpression()
		throws ExpressionException, MalformedExpressionException {
		if (expressions[0] != null) {
			expressions[0].compileExpression();
			if ((ExpressionTypeHelper.isArithmeticType(expressions[0].getType())) ||
				(expressions[0].getType().getTypeCode()==QTypeTuple.TYPECODE_BOOLEAN) ||
				(expressions[0].getType().getTypeCode()==QTypeTuple.TYPECODE_CHAR) ||
				(expressions[0].getType().getTypeCode()==QTypeTuple.TYPECODE_BYTE)) {
				throw new TypingException("IsNull does not work on primitive types");
			}
			else {
				result = new BasicVariableOperand(type);
				verified = true;
			}
		}
		else
		//Operands not yet assigned
			throw new MalformedExpressionException("null children value");
		return result;
	}

	public String getOperatorString() {
		return (not ? "IS NOT NULL" : "IS NULL");
	}
}
