/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */

package org.objectweb.medor.lib;

import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.clone.lib.BasicCloneable;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.util.monolog.api.Logger;

import java.util.Map;

public class BasicField extends BasicCloneable implements Field {
	protected String name;
	protected PType type;
	protected short valueState;
	transient protected Logger logger;

	public BasicField() {
		logger = Log.getLoggerFactory().getLogger(getClass().getName());
	}

	public BasicField(String name) {
		this();
		this.name = name;
	}

	public BasicField(String name, PType type) {
		this(name);
		this.type = type;
	}

	public BasicField(String name, PType type, short valueState)
		throws MedorException {
		this(name, type);
		if ((valueState != Field.NONULLS) &&
			(valueState != Field.NULLABLE) &&
			(valueState != Field.NULLABLEUNKNOWN)) {
			throw new MedorException("invalid nulStatus Field Error: " + valueState);
		}
		this.valueState = valueState;
	}

	public Object clone(Object clone,
						Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		((BasicField) clone).name = name;
		((BasicField) clone).type = type;
		((BasicField) clone).valueState = valueState;
		return clone;
	}

	public String getName() {
		return name;
	}

	public PType getType() {
		return type;
	}

	public short getNullStatus() {
		return valueState;
	}

	public String toString() {
		return name;
	}
}
