/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.lib;

import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.clone.api.Cloneable;
import org.objectweb.medor.clone.lib.BasicCloneable;

import org.objectweb.util.monolog.api.Logger;

import java.util.HashMap;
import java.util.ArrayList;
import java.util.Map;

public class BasicTupleStructure extends BasicCloneable implements TupleStructure {

	protected ArrayList fields = new ArrayList();
    protected HashMap name2field = new HashMap();
    transient protected Logger logger;

	protected BasicTupleStructure() {
		logger = Log.getLoggerFactory().getLogger(getClass().getName());
	}

    public BasicTupleStructure(Field[] fields) throws MedorException {
		this();
        for (int cpt = 0; cpt < fields.length; cpt++) {
			// strores the fields indexed by rank
            this.fields.add(fields[cpt]);
            // strores the fields indexed by name
            if (name2field.put(fields[cpt].getName(), fields[cpt]) != null) {
				throw new MedorException("Duplicated Field Name: " + fields[cpt].getName());
			}
        }
    }

	public Object clone(Object clone,
						Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		BasicTupleStructure bts = (BasicTupleStructure) clone;
		bts.fields = new ArrayList(fields.size());
		bts.name2field = new HashMap();
		for(int i=0; i<fields.size(); i++) {
			Field f = (Field) getClone((Cloneable) fields.get(i), obj2clone);
			bts.fields.add(f);
			bts.name2field.put(f.getName(), f);
		}
		return clone;
	}

    public Field[] getFields() {
        return (Field[]) fields.toArray(new Field[fields.size()]);
    }

    public Field getField(String fieldname) throws MedorException {
        Field res = (Field) name2field.get(fieldname);
		if (res==null) {
            throw new MedorException("Field name error: " + fieldname);
        }
		return res;
    }

    public Field getField(int fieldrank) throws MedorException {
        Field f = (Field) fields.get(fieldrank - 1);
        if (f == null) {
            throw new MedorException("Field rank error : " + fieldrank);
        } else {
            return f;
        }
    }

    public int getFieldRank(Field f) throws MedorException {
        int idx = fields.indexOf(f);
		if (idx == -1)
			throw new MedorException("Field not found: " + f.getName());
		else return idx+1;
    }


    public int getSize() {
        return fields.size();
    }

	public boolean contains(Field f) {
		return fields.contains(f);
	}

    public boolean contains(String fieldName) {
        return name2field.containsKey(fieldName);
    }
}
