/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.optim.jorm;

import org.objectweb.medor.lib.Log;
import org.objectweb.medor.optim.api.LeafRewriteRule;
import org.objectweb.medor.optim.api.LeafRewriter;
import org.objectweb.medor.optim.lib.BasicQueryRewriter;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

/**
 * This QueryRewriter is able to rewrite a query tree which
 * contains JORM leaves.
 *
 * @author S.Chassande-Barrioz
 */
public class JormQueryRewriter extends BasicQueryRewriter {

    /**
     * It builds a QueryRewriter which the configuration is composed by the rule
     * needed to rewrite a query tree containing jorm leaves (ClassExtent or
     * GenClassExtent).
     * @param jormName2Mapper is map which associated the jorm leaves to their
     * mapper instance.
	 * @param jormName2Project is map which associated the jorm leaves to their
	 * project name.
     * @param leafRewriters is a collection of LeafRewriter which knows how to
	 * rewrite the JORM leaves.
     * These LeafRewriter depend on the mapping used by the Jorm leaves
     */
    public JormQueryRewriter(Map jormName2Mapper, Map jormName2Project,
                             Collection leafRewriters) {
        super();
		log = Log.loggerFactory.getLogger(
			Log.MEDOR_PREFIX + ".optim.rewriter.JormQueryRewriter");
        rules = getDefaultRules();

        JormAssignMapperRule am = new JormAssignMapperRule();
        am.setJormName2Mapper(jormName2Mapper,  jormName2Project);
        rules.add(0, am);

        LeafRewriteRule lrr = new JormLeafRewriteRule();
        for (Iterator it = leafRewriters.iterator(); it.hasNext();)
            lrr.addLeafRewriter((LeafRewriter) it.next());
        rules.add(1, lrr);
        rules.add(2, new JormGoUpDecodeRule());
    }

    /**
     * It builds a QueryRewriter which the configuration is composed by the rule
     * needed to rewrite a query tree containing jorm leaves (ClassExtent or
     * GenClassExtent).
     * @param leafRewriters is a collection of LeafRewriter which knows how to
	 * rewrite the JORM leaves.
     * These LeafRewriter depend on the mapping used by the Jorm leaves
     */
    public JormQueryRewriter(Collection leafRewriters) {
        super();
		log = Log.loggerFactory.getLogger(
			Log.MEDOR_PREFIX + ".optim.rewriter.JormQueryRewriter");
        rules = getDefaultRules();
        LeafRewriteRule lrr = new JormLeafRewriteRule();
        for (Iterator it = leafRewriters.iterator(); it.hasNext();)
            lrr.addLeafRewriter((LeafRewriter) it.next());
        rules.add(0, lrr);
        rules.add(1, new JormGoUpDecodeRule());
    }
}
