/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

/**
 * Package definition.
 */

package org.objectweb.medor.optim.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.expression.api.BinaryArithmeticOperator;
import org.objectweb.medor.expression.api.BinaryLogicalOperator;
import org.objectweb.medor.expression.api.Comparator;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.UnaryArithmeticOperator;
import org.objectweb.medor.expression.api.UnaryLogicalOperator;
import org.objectweb.medor.filter.api.BelongOperator;
import org.objectweb.medor.filter.api.ExpressionHelper;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.postfix.PostfixExpressionHelper;
import org.objectweb.medor.optim.api.ExecPlanGenerator;
import org.objectweb.medor.query.api.CalculatedField;
import org.objectweb.medor.query.api.QueryNode;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.tuple.api.TupleLoader;
import org.objectweb.medor.type.lib.QTypeTuple;
import org.objectweb.util.io.api.Block;
import org.objectweb.util.io.api.TargetHolder;
import org.objectweb.util.io.lib.BlockImpl;
import org.objectweb.util.io.lib.JavaFileHolder;

import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Stack;
import java.util.Vector;

public class SourceCodeGeneratorCompiler implements ExecPlanGenerator {
    private DirsCompileClassLoader mcl;
    private TargetHolder queryFiltersHolder;
    private TargetHolder tcTypeHolder;
    private QueryTree queryTree;
    private String sourceDirectory,classDirectory;
    private HashSet library = new HashSet();
    ExpressionHelper eHelper = new PostfixExpressionHelper();
    private int nbrFilter = 0;
    private int nbrQTType = 0;
    private static final String javaFilterName = "QueryFilter";
    private static final String tupleLoaderClassName = "TupleLoader";

    public SourceCodeGeneratorCompiler(String outputSourceDirectory,
                                       String outputClassdirectory,
                                       String[] libraryPaths)
            throws MedorException {
        try {
            sourceDirectory = outputSourceDirectory;
            classDirectory = outputClassdirectory;
            File tadir;
            tadir = new File(sourceDirectory);
            if (!tadir.isDirectory()) {
                throw new MedorException("the output directory parametre must be a directory:  " + sourceDirectory);
            }
            library.add(sourceDirectory);

            tadir = new File(classDirectory);
            if (!tadir.isDirectory()) {
                throw  new MedorException("the output directory parametre must be a directory:  " + classDirectory);
            }
            library.add(classDirectory);

            for (int i = 0; (i < libraryPaths.length); i++) {
                File targetDirectory = new File(libraryPaths[i]);
                if (!targetDirectory.exists()) {
                    throw new MedorException("the path library  " + libraryPaths[i] + "does not exist");
                }
                library.add(libraryPaths[i]);
            }
            queryFiltersHolder = new JavaFileHolder(sourceDirectory);
            tcTypeHolder = new JavaFileHolder(sourceDirectory);
        } catch (Exception e) {
            throw new MedorException(e.getMessage());
        }
    }

    public void buildFilterSources(QueryTree query) throws MedorException {
        if ((query instanceof QueryNode)
                && (((QueryNode) query).getQueryFilter() != null)) {
            QueryNode qn = (QueryNode) query;
            generateFilterSourceCode(qn.getQueryFilter(),
                    javaFilterName + nbrFilter++);
            QueryTree[] qts = qn.getChildren();
            for (int cpt = 0; (cpt < qts.length); cpt++)
                buildFilterSources(qts[cpt]);
        }
    }

    public void buildTupleLoader(QueryTree query) throws MedorException {
        //try {
        if ((query instanceof QueryNode)
                && ((QueryNode) query).getTupleLoader() != null) {
            generateTupleLoader(
                    ((QueryNode) query).getTupleLoader().getFieldIndexes(),
                    query.getTupleStructure(),
                    tupleLoaderClassName + nbrQTType++);
            QueryTree[] qts = ((QueryNode) query).getChildren();
            for (int cpt = 0; (cpt < qts.length); cpt++)
                buildTupleLoader(qts[cpt]);
        }
        //} catch (Exception e) {
        //   throw new MedorException(e.getMessage());
        //}
    }

    public QueryTree getQueryTree() {
        return queryTree;
    }

    public String[] getOutPutDirectoryName() {
        String[] dirs = new String[2];
        dirs[0] = sourceDirectory;
        dirs[1] = classDirectory;

        return dirs;
    }

    /**
     * Generate a typed QueryFilter corresponding to the old QueryTree's
     * QueryFilter
     * @throws MedorException if IO error
     */
    private void buildAllQueryFilter() throws MedorException {
        buildFilterSources(queryTree);
    }

    private void buildAllTupleLoader() throws MedorException {
        buildTupleLoader(queryTree);
    }

    /*
     * This method generate a java class code only for this QueryTree.
     * @param : QueryTree
     */
    private void generateTupleLoader(int[] indexes,
                                     TupleStructure ts,
                                     String javaFileName)
            throws MedorException {
        try {
            FileWriter fw = tcTypeHolder.getFileWriter(javaFileName + ".java");
            byte b = 0;
            Block bk = new BlockImpl(fw, b);
            bk.writeLine("/**");
            bk.writeLine(" * This class is generated by Medor Compiler");
            bk.writeLine(" * It represents a TupleLoader Object");
            bk.writeLine(" */");
            bk.newLine();
            bk.writeLine("package org.objectweb.medor.filter.lib;");
            bk.newLine();
            bk.writeLine("import org.objectweb.medor.api.MedorException;");
            bk.writeLine("import org.objectweb.medor.tuple.api.TupleLoader;");
            bk.writeLine("import org.objectweb.medor.tuple.api.Tuple;");
            bk.writeLine("import org.objectweb.medor.expression.api.ParameterOperand;");
            bk.writeLine("import org.objectweb.medor.filter.api.VariableOperand;");
            bk.writeLine("import org.objectweb.medor.api.TupleStructure;");
            bk.newLine();
            ArrayList listParams = new ArrayList();
            bk.openBlock("public final class " + javaFileName + " implements TupleLoader");
            bk.newLine();
            for (int i = 1; (i <= ts.getSize()); i++) {
                if (ts.getField(i) instanceof CalculatedField) {
                    String[] parameters = eHelper.getParameterNames(((CalculatedField) (ts.getField(i))).getExpression());
                    for (int cpt = 0; (cpt < parameters.length); cpt++) {
                        bk.writeLine("ParameterOperand " + parameters[cpt] + ";");
                        listParams.add(parameters[cpt]);
                    }
                }
            }
            String[] allParameters = new String[listParams.size()];
            for (int cpt = 0; (cpt < listParams.size()); cpt++) {
                allParameters[cpt] = (String) listParams.get(cpt);
            }
            if (allParameters.length > 0) {
                bk.newLine();
                bk.openBlock("private int getParamRank(String paramName, ParameterOperand[] parameters) throws MedorException");
                bk.writeLine("int i;");
                bk.writeLine("for (i=0; (i<parameters.length && !(parameters[i].getName().equalsIgnoreCase(paramName)));i++);");
                bk.writeLine("if (i== parameters.length) throw new MedorException (\"Parameter Error\" + paramName);");
                bk.writeLine("return i;");
                bk.closeBlock();
            }
            bk.newLine();
            bk.openBlock("public void loadTuple(Tuple thisTuple, VariableOperand[] temp, ParameterOperand[] parameters) throws MedorException");
            for (int cpt = 0; (cpt < allParameters.length); cpt++) {
                bk.writeLine(allParameters[cpt] + "= parameters[getParamRank(" + " \"" + allParameters[cpt] + "\" ,parameters)];");
            }

            PType type;
            int size = indexes.length;
            for (int i = 0; (i < size); i++) {
                //bk.writeLine("VariableOperand temp1 = (VariableOperand) temp[i];
                type = ts.getField(i + 1).getType();
                if (indexes[i] != -1) {
                    switch (type.getTypeCode()) {
                        case QTypeTuple.TYPECODE_INT:
                            bk.writeLine("temp[" + i + "].setValue(thisTuple.getInt(" + indexes[i] + "));");
                            break;
                        case QTypeTuple.TYPECODE_SHORT:
                            bk.writeLine("temp[" + i + "].setValue(thisTuple.getShort(" + indexes[i] + "));");
                            break;
                        case QTypeTuple.TYPECODE_BYTE:
                            bk.writeLine("temp[" + i + "].setValue(thisTuple.getByte(" + indexes[i] + "));");
                            break;
                        case QTypeTuple.TYPECODE_LONG:
                            bk.writeLine("temp[" + i + "].setValue(thisTuple.getLong(" + indexes[i] + "));");
                            break;
                        case QTypeTuple.TYPECODE_DOUBLE:
                            bk.writeLine("temp[" + i + "].setValue(thisTuple.getDouble(" + indexes[i] + "));");
                            break;
                        case QTypeTuple.TYPECODE_BOOLEAN:
                            bk.writeLine("temp[" + i + "].setValue(thisTuple.getBoolean(" + indexes[i] + "));");
                            break;
                        case QTypeTuple.TYPECODE_FLOAT:
                            bk.writeLine("temp[" + i + "].setValue(thisTuple.getFloat(" + indexes[i] + "));");
                            break;
                        case QTypeTuple.TYPECODE_STRING:
                            bk.writeLine("temp[" + i + "].setValue(thisTuple.getString(" + indexes[i] + "));");
                            break;
                        case QTypeTuple.TYPECODE_CHAR:
                            bk.writeLine("temp[" + i + "].setValue(thisTuple.getChar(" + indexes[i] + "));");
                            break;
                        case QTypeTuple.TYPECODE_DATE:
                            bk.writeLine("temp[" + i + "].setValue(thisTuple.getDate(" + indexes[i] + "));");
                            break;
                            //TODO handle TupleCollections
                        default:
                            if (type.getJormName().equals("collection"))
                                bk.writeLine("temp[" + i + "].setValue(thisTuple.getTupleCollection(" + indexes[i] + "));");
                            break;
                            //default : throw new MedorException(TYPE_1,"Invalid Attribute type " + type);
                    }
                } else {
                    // Loading a CalculatedField
                    Expression calculExp = ((CalculatedField) ts.getField(i + 1)).getExpression();
                    String filterExpression = generateQueryFilterString(eHelper.toStack(calculExp));
                    bk.writeLine("temp[" + i + "].setValue(" + filterExpression + ");");
                }
            }
            bk.closeBlock();
            bk.newLine();
            bk.openBlock("public int[] getFieldIndexes()");
            bk.writeLine("return null;"); // Not called when evaluating
            bk.closeBlock();
            bk.newLine();
            bk.openBlock("public TupleStructure getTupleStructure()");
            bk.writeLine("return null;"); // Not called when evaluating
            bk.closeBlock();
            bk.closeBlock();
            fw.close();
        } catch (Exception e) {
            throw new MedorException(e);
        }
    }

    /*
     * This method generate a java class code only for this QueryFilter.
     * @param filter: PostFixeQueryFilter
     */
    private void generateFilterSourceCode(Expression filter,
                                          String javaFileName) throws MedorException {

        try {
            FileWriter fw = queryFiltersHolder.getFileWriter(javaFileName + ".java");
            byte b = 0;
            Block bk = new BlockImpl(fw, b);
            bk.writeLine("/**");
            bk.writeLine(" * This class is generated by Medor Compiler");
            bk.writeLine(" * It represents a compiled Expression");
            bk.writeLine(" */");
            bk.newLine();
            bk.writeLine("package org.objectweb.medor.filter.lib;"); // Not obliged
            bk.newLine();
            bk.writeLine("import org.objectweb.medor.api.MedorException;");
            bk.writeLine("import org.objectweb.medor.filter.api.Expression;");
            bk.writeLine("import org.objectweb.medor.filter.api.Operand;");
            bk.writeLine("import org.objectweb.medor.filter.api.VariableOperand;");
            bk.writeLine("import org.objectweb.medor.filter.lib.BasicVariableOperand;");
            bk.writeLine("import org.objectweb.medor.expression.api.ParameterOperand;");
            bk.writeLine("import org.objectweb.medor.type.lib.PTypeSpaceMedor;");
            bk.writeLine("import org.objectweb.jorm.type.api.PType;");
            bk.writeLine("import org.objectweb.medor.tuple.api.Tuple;");
            bk.newLine();
            bk.openBlock("public final class " + javaFileName + " implements Expression");
            bk.newLine();
            bk.writeLine("VariableOperand result = new BasicVariableOperand(PTypeSpaceMedor.BOOLEAN);");
            String[] allParameters = eHelper.getParameterNames(filter);
            for (int cpt = 0; (cpt < allParameters.length); cpt++) {
                bk.writeLine("ParameterOperand " + allParameters[cpt] + ";");
            }

            if (allParameters.length > 0) {
                bk.newLine();
                bk.openBlock("private int getParamRank(String paramName, ParameterOperand[] parameters) throws MedorException");
                bk.writeLine("int i;");
                bk.writeLine("for (i=0; (i<parameters.length && !(parameters[i].getName().equalsIgnoreCase(paramName)));i++);");
                bk.writeLine("if (i== parameters.length) throw new MedorException (paramName);");
                bk.writeLine("return i;");
                bk.closeBlock();
            }
            bk.newLine();
            bk.openBlock("public void evaluate(ParameterOperand[] parameters,Tuple thisTuple) throws MedorException");
            for (int cpt = 0; (cpt < allParameters.length); cpt++) {
                bk.writeLine(allParameters[cpt] + "= parameters[getParamRank(" + "\"" + allParameters[cpt] + "\" ,parameters)];");
            }

            String filterExpression = generateQueryFilterString(eHelper.toStack(filter));

            bk.writeLine("result.setValue( " + filterExpression + ");");
            bk.closeBlock();

            String ptype = "";
            bk.newLine();
            bk.openBlock("public PType getType()");
            switch (filter.getType().getTypeCode()) {
                case QTypeTuple.TYPECODE_BYTE:
                    ptype = "PTypeSpaceMedor.BYTE";
                    break;
                case QTypeTuple.TYPECODE_DOUBLE:
                    ptype = "PTypeSpaceMedor.DOUBLE";
                    break;
                case QTypeTuple.TYPECODE_INT:
                    ptype = "PTypeSpaceMedor.INT";
                    break;
                case QTypeTuple.TYPECODE_FLOAT:
                    ptype = "PTypeSpaceMedor.FLOAT";
                    break;
                case QTypeTuple.TYPECODE_LONG:
                    ptype = "PTypeSpaceMedor.LONG";
                    break;
                case QTypeTuple.TYPECODE_SHORT:
                    ptype = "PTypeSpaceMedor.SHORT";
                    break;
                case QTypeTuple.TYPECODE_BOOLEAN:
                    ptype = "PTypeSpaceMedor.BOOLEAN";
                    break;
                case QTypeTuple.TYPECODE_CHAR:
                    ptype = "PTypeSpaceMedor.CHAR";
                    break;
                case QTypeTuple.TYPECODE_DATE:
                    ptype = "PTypeSpaceMedor.DATE";
                    break;
            }
            bk.writeLine("return " + ptype + ";");
            bk.closeBlock();
            bk.newLine();
            bk.openBlock("public boolean isCompiled()");
            bk.writeLine("return true;"); // Not called when evaluating
            bk.closeBlock();
            bk.newLine();
            bk.openBlock("public Operand getResult()");
            bk.writeLine("return result;"); // Not called when evaluating
            bk.closeBlock();
            bk.newLine();
            bk.openBlock("public void compileExpression()");
            bk.closeBlock();
            bk.newLine();
            bk.openBlock("public Object clone()");
            bk.writeLine("return this;");
            bk.closeBlock();

            bk.newLine();
            bk.closeBlock();
            fw.close();
        } catch (Exception e) {
            throw new MedorException(e);
        }
    }

    private String generateQueryFilterString(Expression[] regularExp)
			throws ExpressionException {
        Object[] polonfpf = regularExp;
        Stack popBuffer = new Stack();
        int cpt = 0;
        //PType currentType = null;
        Comparator c;
        BinaryLogicalOperator bLogicop;
        BinaryArithmeticOperator bArithOp;
        Object op1,op2;
        String leftOp,rightOp,opExp = "";
        String operator;

        while (cpt < polonfpf.length) {
            if (polonfpf[cpt] instanceof Operand)
                popBuffer.push(polonfpf[cpt]);
            else if (polonfpf[cpt] instanceof Comparator) {
                op2 = popBuffer.pop();
                op1 = popBuffer.pop();
                c = (Comparator) polonfpf[cpt];
                operator = c.getOperatorString();
                if (op1 instanceof Operand) {
                    leftOp = generateOperandString((Operand) op1);
                    //currentType = ((Operand) op1).getType();
                } else
                    leftOp = ((String) op1);

                if (op2 instanceof Operand) {
                    rightOp = generateOperandString((Operand) op2);
                } else
                    rightOp = ((String) op2);

                if (!c.getType().isa(PTypeSpace.STRING)) {
                    opExp = "(" + leftOp + operator + rightOp + ")";
                } else {
                    opExp = "(" + leftOp + "compareTo(" + rightOp + ")" + operator + "0)";
                }
                popBuffer.push(opExp);

            } else if (polonfpf[cpt] instanceof BelongOperator) {
                op2 = popBuffer.pop();
                op1 = popBuffer.pop();

                if (op1 instanceof Operand) {
                    leftOp = generateOperandString((Operand) op1);
                } else
                    leftOp = ((String) op1);

                if (op2 instanceof Operand) {
                    rightOp = generateOperandString((Operand) op2);
                } else
                    rightOp = ((String) op2);

                opExp = "(new InTupleCollection().evaluate(leftOp , rightOp)";
                popBuffer.push(opExp);

            } else if (polonfpf[cpt] instanceof BinaryLogicalOperator) {
                op2 = popBuffer.pop();
                op1 = popBuffer.pop();
                bLogicop = (BinaryLogicalOperator) polonfpf[cpt];
                operator = bLogicop.getOperatorString();
                if (op1 instanceof Operand) {
                    leftOp = generateOperandString((Operand) op1);
                } else
                    leftOp = ((String) op1);

                if (op2 instanceof Operand) {
                    rightOp = generateOperandString((Operand) op2);
                } else
                    rightOp = ((String) op2);

                opExp = "(" + leftOp + operator + rightOp + ")";
                popBuffer.push(opExp);

            } else if (polonfpf[cpt] instanceof UnaryLogicalOperator) {
                op1 = popBuffer.pop();
                if (op1 instanceof Operand) {
                    leftOp = generateOperandString((Operand) op1);
                } else
                    leftOp = ((String) op1);
                opExp = "(!" + leftOp + ")";
                popBuffer.push(opExp);

            } else if (polonfpf[cpt] instanceof BinaryArithmeticOperator) {
                op2 = popBuffer.pop();
                op1 = popBuffer.pop();
                bArithOp = (BinaryArithmeticOperator) polonfpf[cpt];
                operator = bArithOp.getOperatorString() + "";
                if (op1 instanceof Operand) {
                    leftOp = generateOperandString((Operand) op1);
                } else
                    leftOp = ((String) op1);
                if (op2 instanceof Operand) {
                    rightOp = generateOperandString((Operand) op2);
                } else
                    rightOp = ((String) op2);
                opExp = "(" + leftOp + operator + rightOp + ")";
                popBuffer.push(opExp);

            } else if (polonfpf[cpt] instanceof UnaryArithmeticOperator) {
                UnaryArithmeticOperator unArith = (UnaryArithmeticOperator) polonfpf[cpt];
                op1 = popBuffer.pop();
                if (op1 instanceof Operand) {
                    leftOp = generateOperandString((Operand) op1);
                } else
                    leftOp = ((String) op1);

                opExp = "(" + unArith.getOperatorString() + leftOp + ")";
                popBuffer.push(opExp);
            }
            cpt++;
        }
        return (String) popBuffer.pop();
    }

    private String generateOperandString(Operand op)
			throws ExpressionException {
        String operandExp = "";
        int range;
        PType type = op.getType();
        switch (type.getTypeCode()) {
            case QTypeTuple.TYPECODE_INT:
                if (op instanceof ParameterOperand) {
                    operandExp = ((ParameterOperand) op).getName() + ".getInt()";
                } else if (op instanceof FieldOperand) {
                    range = ((FieldOperand) op).getIndex();
                    operandExp = "thisTuple.getInt(" + range + ")";
                } else
                    operandExp = op.getInt() + "";
                break;
            case QTypeTuple.TYPECODE_SHORT:
                if (op instanceof ParameterOperand) {
                    operandExp = ((ParameterOperand) op).getName() + ".getShort()";
                } else if (op instanceof Operand)
                    operandExp = op.getShort() + "";
                else if (op instanceof FieldOperand) {
                    range = ((FieldOperand) op).getIndex();
                    operandExp = "thisTuple.getShort(" + range + ")";
                }
                break;

            case QTypeTuple.TYPECODE_DATE:
                if (op instanceof ParameterOperand) {
                    operandExp = ((ParameterOperand) op).getName() + ".getByte()";
                } else if (op instanceof Operand)
                    operandExp = op.getByte() + "";
                else if (op instanceof FieldOperand) {
                    range = ((FieldOperand) op).getIndex();
                    operandExp = "thisTuple.getByte(" + range + ")";
                }
                break;

            case QTypeTuple.TYPECODE_LONG:
                if (op instanceof ParameterOperand) {
                    operandExp = ((ParameterOperand) op).getName() + ".getLong()";
                } else if (op instanceof Operand)
                    operandExp = op.getLong() + "";
                else if (op instanceof FieldOperand) {
                    range = ((FieldOperand) op).getIndex();
                    operandExp = "thisTuple.getLong(" + range + ")";
                }
                break;

            case QTypeTuple.TYPECODE_DOUBLE:
                if (op instanceof ParameterOperand) {
                    operandExp = ((ParameterOperand) op).getName() + ".getDouble()";
                } else if (op instanceof Operand)
                    operandExp = op.getDouble() + "";
                else if (op instanceof FieldOperand) {
                    range = ((FieldOperand) op).getIndex();
                    operandExp = "thisTuple.getDouble(" + range + ")";
                }
                break;

            case QTypeTuple.TYPECODE_FLOAT:
                if (op instanceof ParameterOperand) {
                    operandExp = ((ParameterOperand) op).getName() + ".getFloat()";
                } else if (op instanceof Operand)
                    operandExp = op.getFloat() + "";
                else if (op instanceof FieldOperand) {
                    range = ((FieldOperand) op).getIndex();
                    operandExp = "thisTuple.getFloat(" + range + ")";
                }
                break;

            case QTypeTuple.TYPECODE_BOOLEAN:
                if (op instanceof ParameterOperand) {
                    operandExp = ((ParameterOperand) op).getName() + ".getBoolean()";
                } else if (op instanceof Operand)
                    operandExp = op.getBoolean() + "";
                else if (op instanceof FieldOperand) {
                    range = ((FieldOperand) op).getIndex();
                    operandExp = "thisTuple.getBoolean(" + range + ")";
                }
                break;

            case QTypeTuple.TYPECODE_CHAR:
                if (op instanceof ParameterOperand) {
                    operandExp = ((ParameterOperand) op).getName() + ".getChar()";
                } else if (op instanceof Operand)
                    operandExp = op.getChar() + "";
                else if (op instanceof FieldOperand) {
                    range = ((FieldOperand) op).getIndex();
                    operandExp = "thisTuple.getChar(" + range + ")";
                }
                break;

            case QTypeTuple.TYPECODE_STRING:
                if (op instanceof ParameterOperand) {
                    operandExp = ((ParameterOperand) op).getName() + ".getString()";
                } else if (op instanceof Operand)
                    operandExp = op.getString();
                else if (op instanceof FieldOperand) {
                    range = ((FieldOperand) op).getIndex();
                    operandExp = "thisTuple.getString(" + range + ")";
                }
                break;

            default:
                // first case: TupleCollection
                if (type.getJormName().equals("collection")) {
                    if (op instanceof ParameterOperand) {
                        operandExp = ((ParameterOperand) op).getName() + ".getTupleCollection()";
                    } else if (op instanceof Operand)
                        operandExp = op.getObject() + "";
                    else if (op instanceof FieldOperand) {
                        range = ((FieldOperand) op).getIndex();
                        operandExp = "thisTuple.getTupleCollection(" + range + ")";
                    }
                    break;
                } else if (type.getJormName().equals("tuple")) {

                    if (op instanceof ParameterOperand) {
                        operandExp = ((ParameterOperand) op).getName() + ".getTuple()";
                    } else if (op instanceof Operand)
                        operandExp = op.getObject() + "";
                    else if (op instanceof FieldOperand) {
                        range = ((FieldOperand) op).getIndex();
                        operandExp = "thisTuple.getTuple(" + range + ")";
                    }
                    break;
                }
        }

        return operandExp;
    }

    public Iterator getSourceFileIterator() {
        return queryFiltersHolder.iterateFile();
    }

    /**
     * In this case we will load the query filters generated class into the vm
     * and reaffect them to the new QueryTree
     */
    public QueryTree transform(QueryTree query) throws MedorException {
        this.queryTree = query;
        //TODO ajouter l'appel  la vrification des filter et des
        //tuplestructure (voir interface QueryIndexGenerator).
        //parcouring and changing the queryFilters
        buildAllQueryFilter();
        buildAllTupleLoader();
        Vector list = new Vector();
        Iterator i1 = queryFiltersHolder.iterateFile();
        Iterator i2 = tcTypeHolder.iterateFile();
        while (i1.hasNext()) {
            list.add(i1.next());
        }
        while (i2.hasNext()) {
            list.add(i2.next());
        }
        javac(list.elements());
        //QueryTree copy = (QueryTree) queryTree.clone();
        QueryTree copy = queryTree;
        library.add(sourceDirectory);
        mcl = new DirsCompileClassLoader(library);
        nbrFilter = 0;
        nbrQTType = 0;
        compileFilters(copy, javaFilterName + nbrFilter);
        compileTupleLoader(copy, tupleLoaderClassName + nbrQTType);
        //i++;
        return copy;
    }

    private void compileFilters(QueryTree query, String fileName)
            throws MedorException {
        Class c = null;
        try {
            //int i = 0;
            if ((query instanceof QueryNode) &&
                    (((QueryNode) query).getQueryFilter() != null)) {
                nbrFilter++;
                c = mcl.loadClass("org.objectweb.medor.filter.lib." + fileName);
                Object o = c.newInstance();
                Expression predicate = (Expression) o;
                ((QueryNode) query).setQueryFilter(predicate);
                QueryTree[] qts = ((QueryNode) query).getChildren();

                for (int cpt = 0; (cpt < qts.length); cpt++) {

                    compileFilters(qts[cpt], javaFilterName + nbrFilter);

                }
            }
        } catch (ClassNotFoundException exp) {
            throw new MedorException("THIS CLASS NOT EXIST TRY TO COMPILE AT FIRST");
        } catch (InstantiationException iexo) {
        } catch (IllegalAccessException e) {
        }
    }

    private void compileTupleLoader(QueryTree query, String fileName)
            throws MedorException {
        Class c = null;
        try {
            //int i = 0;
            if ((query instanceof QueryNode) &&
                    ((QueryNode) query).getTupleLoader() != null) {
                nbrQTType++;
                c = mcl.loadClass("org.objectweb.medor.filter.lib." + fileName);
                Object o = c.newInstance();
                TupleLoader tLoader = (TupleLoader) o;
                ((QueryNode) query).setTupleLoader(tLoader);
                //TODO verifier que c'est un noeud
                QueryTree[] qts = ((QueryNode) query).getChildren();

                for (int cpt = 0; (cpt < qts.length); cpt++) {
                    compileTupleLoader(qts[cpt], tupleLoaderClassName + nbrQTType);
                }
            }
        } catch (ClassNotFoundException exp) {
            throw new MedorException("THIS CLASS May be NOT EXIST TRY TO COMPILE AT FIRST or correct the classpath");
        } catch (InstantiationException iexo) {
        } catch (IllegalAccessException e) {
        }
    }

    /**
     * Java compilation of all generated QueryFilter java Sources
     */
    private void javac(Enumeration listFile) {

        Runtime rt = Runtime.getRuntime();
        Process p = null;
        // Compilation by javac or we can imagine an other compilator
        try {
            System.out.println("Launching javac ...");
            String classpath = "";
            String sourceFiles = sourceDirectory + File.separator;
            Iterator i = library.iterator();
            if (i.hasNext()) classpath = classpath + ((String) i.next());
            while (i.hasNext()) {
                classpath = classpath + File.pathSeparator + ((String) i.next());
            }

            if (listFile.hasMoreElements()) {
                File f = (File) listFile.nextElement();
                String s = f.getName();
                sourceFiles = sourceFiles + s;
            }

            while (listFile.hasMoreElements()) {
                File f = (File) listFile.nextElement();
                String s = f.getName();
                sourceFiles = sourceFiles + " " + sourceDirectory + File.separator + s;// Only For Windows classpath
            }

            String cmd = "javac -classpath " + classpath + " " + "-d " + classDirectory + " " + sourceFiles;
            System.out.println(cmd);
            try {
                p = rt.exec(cmd);
            } catch (java.io.IOException e) {
                System.err.println(" Compilation failed... " + e.getMessage());
            }
            p.waitFor();
            System.out.println("Compilation finisched ...");

        } catch (InterruptedException e) {
            System.err.println(" Compilation failed... " + e.getMessage());
        }
    }

    public void addLibraryPath(String classpath) throws MedorException {

        File tadir = new File(classpath);
        if (!tadir.exists()) {
            throw  new MedorException("the path library  " + classpath + "does not exist");
        }
        library.add(classpath);
    }

    public Iterator getPathsIterator() {
        return library.iterator();
    }
}
