/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */
package org.objectweb.medor.query.jorm.api;

import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.datasource.api.DataStore;

/**
 * This class represent a a jorm query leaf.
 *
 * @author S.Chassande-Barrioz
 */
public interface JormExtent extends QueryLeaf {

    PMapper getPMapper();

    void setPMapper(PMapper m, String projectName);

    String getProjectName();

    void setProjectName(String projectName);

    String getPNameFieldName();

    void setDataStore(DataStore ds);

    String getJormName();

    JormField getIdentifierField();

    MetaObject getMetaObject();

	/**
	 * Tells whether the current Extent should also contain objects from the
	 * subclasses.
	 * <p>The default is that only objects strictly in the current class are
	 * included, and not objects of the subclasses.
	 * @return true if objects from subclasses are to be included, false if
	 * only strict instances of the corresponding class should be included.
	 */
	boolean withSubClasses();

	/**
	 * Indicates whether the current Extent should also contain objects from the
	 * subclasses.
	 * <p>The default is that only objects strictly in the current class are
	 * included, and not objects of the subclasses.
	 * @param includeSubClasses is true if objects from subclasses are to be
	 * included, false if only strict instances of the corresponding class
	 * should be included.
	 */
	void setWithSubClasses(boolean includeSubClasses);
}
