/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.jorm.lib;

import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.jorm.api.JormField;
import org.objectweb.medor.query.lib.BasicQueryTreeField;

import java.util.Map;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class BasicJormField extends BasicQueryTreeField implements JormField {

	protected TypedElement te;

	public BasicJormField() {
	}

    public BasicJormField(String fieldName, QueryTree qt)
		throws MedorException {
		super(fieldName, qt);

	}
    public BasicJormField(String fieldName, QueryTree qt, TypedElement te)
        throws MedorException {
        super(fieldName, te.getType(), qt);
        this.te = te;
    }

	public BasicJormField(QueryTree qt, TypedElement te)
		throws MedorException {
		//Fields of a JORM Class are all TypedElements, and all
		//have a Name and a PType.
		super(te.getName(), te.getType(), qt);
        this.te = te;
	}

	public Object clone(Object clone,
						Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		((BasicJormField) clone).te = te;
		return clone;
	}

	public TypedElement getTypedElement() {
		return te;
	}
}
