/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.jorm.lib;

import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;

import java.util.Iterator;
import java.util.Map;

/**
 * This class represents the extent of a JORM GenClass.
 * It contains and implements its associated TupleStructure.
 */
public class GenClassExtent extends BasicJormExtent {

	private GenClassRef gcr;

    private String elementFieldName;

	public GenClassExtent() {
	}

	public GenClassExtent(String _name, String pnFieldName) {
		super(_name, pnFieldName);
	}

	/**
	 * Constructs an extent for the JORM GenClass, in the form of a QueryLeaf.
	 * @param gcr is the JORM GenClass meta information object
	 * @param _name is the name of the node (null value => empty string)
	 * @param pnameFieldName is the name of the field representing the PName
	 * of the GenClass id.
	 */
	public GenClassExtent(GenClassRef gcr,
						  String _name,
						  String pnameFieldName,
                          String elemName) throws MedorException {
        super(_name, pnameFieldName);
		this.gcr = gcr;

		//Add the Genclass PName
		identifier = new PNameField(getFieldName(name, pnFieldName), gcr, true, this);
		fields.add(identifier);
		name2field.put(identifier.getName(), identifier);

		// Add the element
        Field f = null;
        elementFieldName = getFieldName(name, elemName);
		if (gcr.isClassRef()) {
            f = new PNameField(elementFieldName, gcr.getClassRef(), this);
        } else if (gcr.isGenClassRef()) {
            f = new PNameField(elementFieldName, gcr.getGenClassRef(), false, this);
        } else if (gcr.isPrimitive()) {
			f = new BasicJormField(elementFieldName, this, gcr.getPrimitiveElement());
		}
		fields.add(f);
		name2field.put(f.getName(), f);

		// Add the indexes
		for (Iterator idxIt = gcr.getIndexFields().iterator();
			 idxIt.hasNext();) {
			TypedElement te1 = (TypedElement) idxIt.next();
			f = new BasicJormField(
				getFieldName(name, te1.getName()), this, gcr);
			fields.add(f);
			name2field.put(f.getName(), f);
		}
	}

	public Object clone(Object clone,
						Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		((GenClassExtent) clone).gcr = gcr;
		((GenClassExtent) clone).elementFieldName = elementFieldName;
		return clone;
	}


    public String getElementFieldName() {
        return elementFieldName;
    }

	public MetaObject getMetaObject() {
		return gcr;
	}

    public String getJormName() {
		return gcr.getGenClassId();
    }

}
