/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.query.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.PropagFromNestedField;
import org.objectweb.medor.api.Field;

import java.util.Map;

/**
 * @author S.Chassande-Barrioz, A.Lefebvre
 */
public class BasicPropagFromNestedField
    extends BasicPropagatedField
    implements PropagFromNestedField {


    protected Field[] ancs;

	public BasicPropagFromNestedField() {
	}

    public BasicPropagFromNestedField(String name,
                                      PType type,
                                      QueryTree qt,
                                      Field[] parents,
                                      Field[] ancestors) throws MedorException {
        super(name, type, qt, parents);
        ancs = ancestors;
    }

	public Object clone(Object clone,
						Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		BasicPropagFromNestedField bpfnf = (BasicPropagFromNestedField) clone;
		if (ancs != null) {
			bpfnf.ancs = new Field[ancs.length];
			for(int i=0; i<ancs.length; i++) {
				bpfnf.ancs[i] = (Field) getClone(ancs[i], obj2clone);
			}
		}
		return clone;
	}

    public Field[] getPreviousFlatField() {
        return ancs;
    }
}
