/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.clone.api.Cloneable;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.query.api.PropagatedField;
import org.objectweb.medor.query.api.QueryTree;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class BasicPropagatedField extends BasicQueryTreeField
	implements PropagatedField {

	protected HashMap ancestors = null;
    protected int index = -1;

	public BasicPropagatedField() {
	}

    public BasicPropagatedField(String name,
                                PType type,
								QueryTree qt,
                                Field[] _ancestors) throws MedorException {
		super(name, type, qt);
        if (_ancestors != null && _ancestors.length > 0) {
            ancestors = new HashMap(_ancestors.length);
            for (int i = 0; i < _ancestors.length; i++) {
                Field[] el = (Field[]) ancestors.get(_ancestors[i].getName());
                if (el == null) {
                    el = new Field[1];
                    el[0] = _ancestors[i];
                    ancestors.put(_ancestors[i].getName(), el);
                } else {
                    Field[] neo = new Field[el.length + 1];
                    System.arraycopy(el, 0, neo, 0, el.length);
                    neo[el.length] = _ancestors[i];
                    ancestors.put(_ancestors[i].getName(), neo);
                }
            }
        }
    }

	public Object clone(Object clone,
						Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		((BasicPropagatedField) clone).index = index;
		if (ancestors != null) {
			BasicPropagatedField bpf = (BasicPropagatedField) clone;
			bpf.ancestors = new HashMap(ancestors.size());
			for(Iterator it = bpf.ancestors.entrySet().iterator(); it.hasNext();) {
				Map.Entry me = (Map.Entry) it.next();
				Field[] as = (Field[]) me.getValue();
				if (as == null) {
					me.setValue(null);
				} else {
					Field[] newAncestors = new Field[as.length];
					for(int i=0; i<as.length; i++) {
						newAncestors[i] = (Field) getClone(as[i], obj2clone);
					}
					me.setValue(newAncestors);
				}
			}
		}

		return clone;
	}

	// IMPLEMENTATION OF THE PropagatedField INTERFACE //
    //-------------------------------------------------//
    public int getIndex() {
        return index;
    }

    public void setIndex(int idx) {
        index = idx;
    }

    public Field[] getPreviousFields() {
        return (Field[]) (ancestors == null
                ? new Field[0]
                : getPrevious().toArray(new Field[0]));
    }

    public Field[] getPreviousFields(String oldFieldName) {
        return (Field[]) (ancestors == null
                ? new Field[0]
                : ancestors.get(oldFieldName));
    }

    public Field[] getOriginFields() {
        if (ancestors == null)
            return new Field[0];

        return (Field[]) getOriginFields(getPrevious(), new ArrayList())
                .toArray(new Field[0]);
    }

    protected ArrayList getOriginFields(ArrayList src, ArrayList dst) {
        ArrayList res = dst;
        for (int i = 0; i < src.size(); i++) {
            Field f = (Field) src.get(i);
            if (f instanceof BasicPropagatedField) {
                res = ((BasicPropagatedField) f).getOriginFields(
                        ((BasicPropagatedField) f).getPrevious(), res);
            } else if (f instanceof BasicCalculatedField) {
                res = ((BasicCalculatedField) f).getOriginUsedFields(
                        ((BasicCalculatedField) f).getUsed(), res);
            } else {
                res.add(f);
            }
        }
        return res;
    }

    protected ArrayList getPrevious() {
        ArrayList res = new ArrayList(ancestors.size());
        Iterator it = ancestors.values().iterator();
        while (it.hasNext()) {
            Field[] current = (Field[]) it.next();
            for (int i = 0; i < current.length; i++) {
                res.add(current[i]);
            }
        }
        return res;
    }

    public short getNullStatus() {
        // TO DO
        return 99;
    }

	public void replacePreviousField(Field[] _ancestors) {
        ancestors.clear();
        for (int i = 0; i < _ancestors.length; i++) {
            Field[] el = (Field[]) ancestors.get(_ancestors[i].getName());
            if (el == null) {
                el = new Field[1];
                el[0] = _ancestors[i];
                ancestors.put(_ancestors[i].getName(), el);
            } else {
                Field[] neo = new Field[el.length + 1];
                System.arraycopy(el, 0, neo, 0, el.length);
                neo[el.length] = _ancestors[i];
                ancestors.put(_ancestors[i].getName(), neo);
            }
        }
	}
}
