/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.rdb.api;

import org.objectweb.medor.clone.api.Cloneable;
/**
 * A QualifiedTable represents a table name with its potential alias.
 * It represents a table used in a SQL request.
 * It is used by RdbExpQueryLeaves and RdbExpFields.
 *
 * @see org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf
 * @see org.objectweb.medor.query.rdb.api.RdbExpField
 *
 * @author	<A HREF="mailto:alia.mourad@rd.francetelecom.com><b>
 *			Mourad Alia
 *			</b></A>
 *		<A HREF="mailto:alexandre.lefebvre@rd.francetelecom.com><b>
 *			Alexandre Lefebvre
 * 			</b></A>
 */

public interface QualifiedTable extends Cloneable {

	/**
	 * Returns the table name for the current QualifiedTable.
	 * @return the table name for the current QualifiedTable.
	 */
	String getTableName();

	/**
	 * Return the alias name for the current QualifiedTable.
	 * @return the alias name for the current QualifiedTable.
	 */
	String getAliasName();

    void setAliasName(String newName);
}
