/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.medor.query.rdb.lib;

import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.query.lib.BasicQueryTree;
import org.objectweb.medor.query.rdb.api.RdbQueryLeaf;
import org.objectweb.medor.query.api.OptimizationMetaData;

import java.util.Map;

/**
 *
 * @author S.Chassande-Barrioz
 */
public abstract class BasicRdbQueryLeaf
	extends BasicQueryTree
	implements RdbQueryLeaf {

	protected boolean isSubquery = false;
	protected Expression sqlFilter;
	protected String query;
	protected DataStore ds;

	public BasicRdbQueryLeaf() {
	}

	public BasicRdbQueryLeaf(String _name,
							 OptimizationMetaData optimizationMD,
							 DataStore ds) {
		super(_name, optimizationMD);
		this.ds = ds;
	}

	public BasicRdbQueryLeaf(String _name,
							 OptimizationMetaData optimizationMD,
							 DataStore ds,
							 String query) {
		super(_name, optimizationMD);
		this.ds = ds;
		this.query = query;
	}

	public Object clone(Object clone,
						Map obj2clone) throws CloneNotSupportedException {
		clone = super.clone(clone, obj2clone);
		BasicRdbQueryLeaf ql = (BasicRdbQueryLeaf) clone;
		ql.sqlFilter = (Expression) getClone(sqlFilter, obj2clone);
		ql.ds = ds;
		ql.query = query;
		ql.isSubquery = isSubquery;
		return clone;
	}
	public boolean isSubquery() {
		return isSubquery;
	}

	public void setIsSubquery(boolean subquery) {
		this.isSubquery = subquery;
	}

	public DataStore getDataStore() {
		return ds;
	}

	public void setDataStore(DataStore ds) {
		this.ds = ds;
	}
}
