/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2004 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, S. Chassande-Barrioz, A. Lefebvre
 */
package org.objectweb.medor.query.rdb.lib;

import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.UnaryOperator;
import org.objectweb.medor.expression.converter.rdb.Expression2WhereClauseImpl;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.jorm.lib.IsNullPName;
import org.objectweb.medor.filter.lib.InCollection;
import org.objectweb.medor.filter.lib.IsNull;
import org.objectweb.medor.filter.lib.MemberOf;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.rdb.api.RdbExpField;

import java.util.Collection;
import java.util.Iterator;

/**
 * This class overrides Expression2WhereClauseImpl for MEDOR-specific operators.
 * <p>It is meant to be used for a particular QueryLeaf.<p>
 *
 * @author A. Lefebvre
 */
public class MedorExpression2WhereClause extends Expression2WhereClauseImpl {

	/**
	 * The QueryLeaf associated to this instance of MedorExpression2WhereClause.
	 */
	private BasicRdbExpQueryLeaf queryLeaf;
	/**
	 * ParameterOperands associated to a particular conversion.
	 */
	private ParameterOperand[] pos;

	public String convertExp2WhereClause(Expression exp,
										 RdbAdapter rdbAdapter,
										 BasicRdbExpQueryLeaf ql,
										 ParameterOperand[] pos)
	throws ExpressionException {
		this.queryLeaf = ql;
		this.pos = pos;
		return convertExp2WhereClause(exp, rdbAdapter);
	}

	public void convertExp2WhereClause(Expression exp, RdbAdapter rdbAdapter, StringBuffer sb)
			throws ExpressionException {
		if (exp instanceof FieldOperand) {
			sb.append(BasicRdbExpQueryLeaf.getQualifiedFieldName(
					((RdbExpField) ((FieldOperand) exp).getField())));
		} else if (exp instanceof IsNullPName) {
			sb.append("(? = ");
			convertExp2WhereClause(new BasicOperand(true), rdbAdapter, sb);
			sb.append(")");
		} else if (exp instanceof InCollection) {
			InCollection bo = (InCollection) exp;
			sb.append("(");
			convertExp2WhereClause(bo.getExpression(0), rdbAdapter, sb);
			sb.append(" ");
			sb.append(toSqlOperator(bo.getOperatorString()));
			sb.append(" ");
			getSQLEnumeration(bo.getExpression(1),
							bo.getElemType(),
							pos, sb);
			sb.append(")");
		} else if (exp instanceof MemberOf) {
			MemberOf mo = (MemberOf) exp;
			int size = mo.getOperandNumber() / 2;
			sb.append("(");
			for (int i = 0; i < size; i++) {
				convertExp2WhereClause(mo.getExpression(i), rdbAdapter, sb);
				sb.append(i < (size - 1) ? ", " : "");
			}
			sb.append(" IN (");
			FieldOperand fop = (FieldOperand) mo.getExpression(size);
			QueryTreeField qtf = (QueryTreeField) fop.getField();
			BasicRdbExpQueryLeaf qLeaf =
					(BasicRdbExpQueryLeaf) qtf.getQueryTree();
			if (qLeaf == queryLeaf) {
				throw new ExpressionException("The field '" + qtf.getName()
						+ "' is not attched to a sub queryleaf (qtf= "
						+ qtf + ")");
			}
			qLeaf.setRdbAdapter(rdbAdapter);
			try {
				sb.append(qLeaf.getSqlRequest(null));
			}
			catch (MedorException e) {
				throw new ExpressionException(e);
			}
			sb.append("))");
		}
		//TODO: in the case of NOT xyz, it may sometimes be better to
		//produce something else, e.g. NOT (toto IS NULL) -> toto IS NOT NULL
		else if (exp instanceof IsNull) { //specific MEDOR
			convertExp2WhereClause(((UnaryOperator) exp).getExpression(0), rdbAdapter, sb);
			sb.append(" ");
			sb.append(toSqlOperator(((Operator) exp).getOperatorString()));
		} else
			super.convertExp2WhereClause(exp, rdbAdapter, sb);
	}

	private void getSQLEnumeration(
			Expression e,
			PType elemType,
			ParameterOperand[] pos,
			StringBuffer sb)
			throws ExpressionException {
		Collection c = null;
		sb.append("(");
		if (e instanceof ParameterOperand) {
			String pn = ((ParameterOperand) e).getName();
			if (pos == null || pos.length == 0)
				throw new ExpressionException("No Parameter");
			int i = 0;
			while (i < pos.length && !pn.equals(pos[i].getName()))
				i++;
			if (i < pos.length)
				c = (Collection) pos[i].getObject();
			else
				throw new ExpressionException("Parameter " + pn + " not found");
		} else
			c = (Collection) ((Operand) e).getObject();
		Iterator it = c.iterator();
		if (!it.hasNext()) {
			throw new ExpressionException("Empty collection for InCollection operator");
		}
		while (it.hasNext()) {
			Object nextit = it.next();
			//case of ParameterOperand
			if (nextit instanceof ParameterOperand) {
				sb.append("?");
			} else if (elemType.equals(PTypeSpace.STRING)) {
				sb.append("'");
				sb.append(nextit);
				sb.append("'");
			} else {
				sb.append(nextit);
			}
			if (it.hasNext()) {
				sb.append(", ");
			}
		}
		sb.append(")");
	}

	public void convertExp2WhereClauseBuilder(Expression exp,
											  String rdbAdapterVarName,
											  StringBuffer sb)
		throws ExpressionException {
		if (exp instanceof FieldOperand) {
			sb.append(BasicRdbExpQueryLeaf.getQualifiedFieldName(
					((RdbExpField) ((FieldOperand) exp).getField())));
		} else if (exp instanceof IsNullPName) {
			sb.append("(? = ");
			convertExp2WhereClauseBuilder(new BasicOperand(true), rdbAdapterVarName, sb);
			sb.append(")");
		} else if (exp instanceof InCollection) {
			InCollection bo = (InCollection) exp;
			sb.append("(");
			convertExp2WhereClauseBuilder(bo.getExpression(0), rdbAdapterVarName, sb);
			sb.append(" ");
			sb.append(toSqlOperator(bo.getOperatorString()));
			sb.append(" ");
			getSQLEnumeration(bo.getExpression(1),
							bo.getElemType(),
							pos, sb);
			sb.append(")");
		} else if (exp instanceof MemberOf) {
			throw new ExpressionException("MemberOf Operator not yet supported at generation time");
		} else if (exp instanceof IsNull) { //specific MEDOR
			convertExp2WhereClauseBuilder(((Operator) exp).getExpression(0), rdbAdapterVarName, sb);
			sb.append(" ");
			sb.append(toSqlOperator(((Operator) exp).getOperatorString()));
		} else
			super.convertExp2WhereClauseBuilder(exp, rdbAdapterVarName, sb);
	}
}
