/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.type.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;

/**
 * This class defines the extension of PType for Tuples.
 */
public class QTypeTuple extends QType {
	public final static short TYPECODE_TUPLE = 101;

	protected PType[] attributeTypes = null;

	/**
	 * It constructs the types that represent tuples.
	 * @param types The array of PTypes for the tuple attributes.
	 * @param typespace	The type space to which the constructed PType belongs.
	 */
	public QTypeTuple(PType[] types, PTypeSpace typespace) {
		super("tuple", typespace);
		attributeTypes = types;
	}

	/**
	 * Checks whether the current QTypeTuple is a subtype of another PType.
	 * The other PType must be a QTypeTuple.
	 * @param pt The other PType to be compared to.
	 * @return It return true if this PType conforms to the one passed as
	 * parameter.
	 */
	public boolean isa(PType pt) {
		//the other PType must be a QTypeTuple
		if (!(pt instanceof QTypeTuple) ||
			(attributeTypes.length != ((QTypeTuple) pt).attributeTypes.length))
		return false;
		//check on all attribute of both PTypeTuples.
		int i = 0;
		while (i < attributeTypes.length) {
			if (!attributeTypes[i].isa(((QTypeTuple) pt).attributeTypes[i]))
			return false;
		}
		//gone through all attributes with success
		return true;
	}

}
